#!/bin/bash

# -------------------------------------------------------------------
# Copyright (C) 2015 by Intevation GmbH
# Author(s):
# Sascha Wilde <wilde@intevation.de>
# Frank Koormann <frank@intevation.de>

# This program is free software under the GNU GPL (>=v2)
# Read the file COPYING coming with the software for details.
# -------------------------------------------------------------------

# Die on errors!
set -e 

ME=`basename $0`

usage ()
{
  cat <<EOF
$ME ID Name Bundesland

Trägt JMD "Name" mit ID in die Auswertungsdatenbanken von
Trägergruppe, BMFSFJ und Servicebüro JMD ein.
EOF
  exit 0
}

fatal ()
{
  echo >&2 "FATAL: $*"
  exit 23
}


check_user ()
{
  [ `id -nu` == "postgres" ] || fatal "$ME must be run as user postgres"
}

get_port ()
# Args: $1 = ID
# Return DB Port for given ID
{
  case $1 in
    3*)
      echo 5432
      return
      ;;
    2*)
      echo 5433
      return
      ;;
    4*)
      echo 5434
      return
      ;;
    1*)
      echo 5435
      return
      ;;
    *)
      fatal "Could not determine db port for id $1"
      ;;
  esac
}

get_traegergruppe_id ()
# Args: $1 = ID
# Return traegergruppe_id for given ID
{
  case $1 in
    3*)
      echo 3000
      return
      ;;
    2*)
      echo 2000
      return
      ;;
    4*)
      echo 4000
      return
      ;;
    1*)
      echo 1000
      return
      ;;
    *)
      fatal "Could not determine db port for id $1"
      ;;
  esac
}

get_traegergruppe ()
# Args: $1 = ID
# Return traegergruppe for given ID
{
  case $1 in
    3*)
      echo awo
      return
      ;;
    2*)
      echo ejsa
      return
      ;;
    4*)
      echo ib
      return
      ;;
    1*)
      echo kjs
      return
      ;;
    *)
      fatal "Could not determine db port for id $1"
      ;;
  esac
}

get_stdb_name ()
# Args: $1 = ID
# Return name of the Strukturdaten-DB for ID
{
  echo "ka_st${1}_db"
}

get_db_name ()
# Args: $1 = ID
# Return name of the DB for ID
{
  echo "ka_${1}_db"
}

#-----------------------------------------------------------------------
# Main

if [ $# -lt 3 ] ; then
    usage
fi

check_user
jmd_id="$1"
name="$2"
bundesland="$3"

port=`get_port $jmd_id`
db=`get_db_name $jmd_id`
traegergruppe=`get_traegergruppe $jmd_id`
stdb=`get_stdb_name $jmd_id`

psql -p $port ka_$traegergruppe\auswertung102_db -c "INSERT
      INTO fkz_tbl (id, name, fkz, database, bundesland_id) 
        VALUES ($jmd_id, '$name', '$jmd_id', '$db', $bundesland);"
psql -p $port ka_st$traegergruppe\auswertung102_db -c "INSERT
      INTO fkz_tbl (id, name, fkz, database, bundesland_id) 
        VALUES ($jmd_id, '$name', '$jmd_id', '$stdb', $bundesland);"
psql -p $port ka_rc$traegergruppe\auswertung102_db -c "INSERT
      INTO fkz_tbl (id, name, fkz, database, bundesland_id) 
        VALUES ($jmd_id, '$name', '$jmd_id', '$stdb', $bundesland);"

psql -p 5436 ka_bmfsfjauswertung102_db -c "INSERT
      INTO fkz_tbl (id, name, fkz, database, bundesland_id) 
        VALUES ($jmd_id, '$name', '$jmd_id', '$db', $bundesland);"
psql -p 5436 ka_stbmfsfjauswertung102_db -c "INSERT
      INTO fkz_tbl (id, name, fkz, database, bundesland_id) 
        VALUES ($jmd_id, '$name', '$jmd_id', '$stdb', $bundesland);"
psql -p 5436 ka_rcbmfsfjauswertung102_db -c "INSERT
      INTO fkz_tbl (id, name, fkz, database, bundesland_id) 
        VALUES ($jmd_id, '$name', '$jmd_id', '$stdb', $bundesland);"

psql -p 5437 ka_servicestelleauswertung102_db -c "INSERT
      INTO fkz_tbl (id, name, fkz, database, bundesland_id) 
        VALUES ($jmd_id, '$name', '$jmd_id', '$db', $bundesland);"
psql -p 5437 ka_stservicestelleauswertung102_db -c "INSERT
      INTO fkz_tbl (id, name, fkz, database, bundesland_id) 
        VALUES ($jmd_id, '$name', '$jmd_id', '$stdb', $bundesland);"
psql -p 5437 ka_rcservicestelleauswertung102_db -c "INSERT
      INTO fkz_tbl (id, name, fkz, database, bundesland_id) 
        VALUES ($jmd_id, '$name', '$jmd_id', '$stdb', $bundesland);"
