-- Upgrade script for rg_case_document_count_tbl and related things.

-- This is a slightly modified copy of
-- install-coaches/config/evaluation/10documentcount.sql, but the only
-- difference is in the comments.


-- This SET ROLE should not be necessary in an upgrade script, but I
-- don't have time to test whether the permissions will be set correctly
-- without it.
SET ROLE :adm_ka_owner;


CREATE TABLE rg_case_document_count_tbl (
    id SERIAL PRIMARY KEY NOT NULL,
    master_id INTEGER NOT NULL,
    document_count BIGINT NOT NULL,

    FOREIGN KEY (master_id) REFERENCES master_tbl (id) ON DELETE CASCADE
);


CREATE OR REPLACE VIEW rg_case_document_count_tbl_anonym_view AS
SELECT id, master_id, document_count
  FROM rg_case_document_count_tbl;

GRANT SELECT ON rg_case_document_count_tbl_anonym_view TO GROUP :cm_group;
GRANT SELECT ON rg_case_document_count_tbl_anonym_view TO GROUP :admin_group;


-- Determine the total number of documents for a given master ID
-- according to rg_case_document_count_tbl_anonym_view. This function
-- can be used in EvaluationSearch to determine the number of documents
-- per agency/FKZ. Due to the aggregation in that search, it's not easy
-- to do this without the function, because sub-SELECTs in parameters of
-- aggregate functions seem to have non-obvious properties (i.e. we
-- could't get it to work in a reasonable amount of time without this
-- function).
CREATE OR REPLACE FUNCTION case_document_count(mid int)
RETURNS BIGINT AS
$$
DECLARE
    count BIGINT;
BEGIN
    SELECT INTO count SUM(r.document_count)
      FROM rg_case_document_count_tbl_anonym_view r
     WHERE r.master_id = mid;

    IF count IS NULL THEN
        count = 0;
    END IF;

    RETURN count;
END;
$$ LANGUAGE plpgsql VOLATILE;


REVOKE ALL    ON FUNCTION case_document_count(int) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION case_document_count(int) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION case_document_count(int) TO GROUP :admin_group;
