-- Only needs to be excuted one time
\connect postgres postgres

UPDATE pg_database SET datallowconn = TRUE where datname='mpuls_template';

\connect mpuls_template postgres
-- Change the password length to 12 in mpuls_template.
-- Needs to replace functions mpuls_create_role and mpuls_reset_role

BEGIN;
CREATE OR REPLACE FUNCTION mpuls_create_role (
        username varchar(64),
        grouprole varchar(64), 
        agency varchar(52)
  ) RETURNS RECORD LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    db_group varchar(64);
    pb_group varchar(64);
    pw       varchar(13);
    result   RECORD;
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to create role %s in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || username);
      IF lower(grouprole) = 'admin' THEN
        db_group := quote_ident('ka_' || agency || '_admin');
      ELSIF lower(grouprole) = 'pb' THEN
        db_group := quote_ident('ka_' || agency || '_pb');
      ELSE
        db_group := quote_ident('ka_' || agency || '_cm');
      END IF;
      pw := mpuls_passgen(12);
      EXECUTE 'CREATE ROLE ' || db_user 
          || ' LOGIN NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE'
          || ' PASSWORD ' || quote_literal(pw);
      EXECUTE 'GRANT ' || db_group || ' TO ' || db_user;
      SELECT INTO result db_name::varchar(64), db_user::varchar(64), pw::varchar(13);
      RETURN result;
    END IF;

  END;
$$ ;
REVOKE ALL ON FUNCTION mpuls_create_role( varchar, varchar, varchar ) 
    FROM public;
GRANT EXECUTE ON FUNCTION mpuls_create_role( varchar, varchar, varchar ) 
    TO admin;

CREATE OR REPLACE FUNCTION mpuls_reset_password (
        name varchar(64),
        agency varchar(52)
  ) RETURNS text LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    pw       varchar(13);
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to reset password for role % in agency % by %.',
          quote_literal(name), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || name);
      pw := mpuls_passgen(12);
      EXECUTE 'ALTER USER ' || db_user || ' WITH PASSWORD ' 
          || quote_literal(pw);
      RETURN pw::text;
    END IF;
    RETURN ''::text;
  END;
$$ ;
REVOKE ALL ON FUNCTION mpuls_reset_password( varchar, varchar ) FROM public;
GRANT EXECUTE ON FUNCTION mpuls_reset_password( varchar, varchar ) 
    TO admin;

COMMIT;

UPDATE pg_database SET datallowconn = FALSE where datname='mpuls_template';

\connect mpuls_maintenance postgres

-- Change the password length to 12 in mpuls_maintenance.
-- Needs to replace functions mpuls_create_role and mpuls_reset_role
BEGIN;
CREATE OR REPLACE FUNCTION mpuls_create_role (
        username varchar(64),
        grouprole varchar(64), 
        agency varchar(52)
  ) RETURNS RECORD LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    db_group varchar(64);
    pb_group varchar(64);
    pw       varchar(13);
    result   RECORD;
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to create role %s in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || username);
      IF lower(grouprole) = 'admin' THEN
        db_group := quote_ident('ka_' || agency || '_admin');
      ELSIF lower(grouprole) = 'pb' THEN
        db_group := quote_ident('ka_' || agency || '_pb');
      ELSE
        db_group := quote_ident('ka_' || agency || '_cm');
      END IF;
      pw := mpuls_passgen(12);
      EXECUTE 'CREATE ROLE ' || db_user 
          || ' LOGIN NOSUPERUSER INHERIT NOCREATEDB NOCREATEROLE'
          || ' PASSWORD ' || quote_literal(pw);
      EXECUTE 'GRANT ' || db_group || ' TO ' || db_user;
      SELECT INTO result db_name::varchar(64), db_user::varchar(64), pw::varchar(13);
      RETURN result;
    END IF;

  END;
$$ ;
REVOKE ALL ON FUNCTION mpuls_create_role( varchar, varchar, varchar ) 
    FROM public;
GRANT EXECUTE ON FUNCTION mpuls_create_role( varchar, varchar, varchar ) 
    TO admin;

CREATE OR REPLACE FUNCTION mpuls_reset_password (
        name varchar(64),
        agency varchar(52)
  ) RETURNS text LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
    pw       varchar(13);
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to reset password for role % in agency % by %.',
          quote_literal(name), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || name);
      pw := mpuls_passgen(12);
      EXECUTE 'ALTER USER ' || db_user || ' WITH PASSWORD ' 
          || quote_literal(pw);
      RETURN pw::text;
    END IF;
    RETURN ''::text;
  END;
$$ ;
REVOKE ALL ON FUNCTION mpuls_reset_password( varchar, varchar ) FROM public;
GRANT EXECUTE ON FUNCTION mpuls_reset_password( varchar, varchar ) 
    TO admin;

COMMIT;
