# -*- coding: utf-8 -*-
#
# Copyright 2018 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls_S (CoMPUter-based case fiLeS -
# Deutschlandstipendium).
#
# mpuls_S is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls_S is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls_S. If not, see <http://www.gnu.org/licenses/>.

import logging

from mpulsweb.controllers.formularpage import FormularpageController as \
    MpulsFormularpageController, PageRenderer
from mpulsweb.lib.formedrenderer import ViewRenderer as MpulsViewRenderer, \
    ViewPrintRenderer as MpulsViewPrintRenderer
from mpulsweb.model.agencysettings import Agency
from formed.renderer.html import Item, ErrorRenderer
from mpulsweb.lib.base import g


log = logging.getLogger(__name__)


def collect_bool_group_warnings(warnings):
    """Turn warnings for check boxes into group-level warnings for bool-groups.

    The bool-groups for which the required-rules are generated
    app_globals have such rules for each individual checkbox in the
    group. This leads to numerous identical warnings when the form is
    initially shown. This function collects all such warnings and
    replaces them with warnings that refer to the group containing the
    checkboxes.
    """
    warnings = warnings.copy()
    for key, messages in warnings.items():
        ncname, ids = key.split(":", 1)
        expected_message = g.bool_group_messages.get(ncname)
        if expected_message in messages:
            # There's a warning about a missing required field for one
            # of our boolean groups. Remove this particular warning and
            # insert a new one for the group itself.
            warnings[key] = [msg for msg in messages if msg != expected_message]
            parent = g.formedTree.name2nc[ncname].getParent()
            warnings[parent.getName() + ":" + ids] = [expected_message]

    return warnings


class CoachesPageRenderer(PageRenderer):

    """Respekt-Coaches specific renderer that handles bool-group warnings.
    """

    def set_errorrenderer(self, errorrenderer, **kw_args):
        def wrap_error_renderer(data, meta_tree, errors, warnings,
                                notices, widget_order, url_factory,
                                languages=None):
            return errorrenderer(data, meta_tree, errors,
                                 collect_bool_group_warnings(warnings), notices,
                                 widget_order, url_factory, languages=languages)

        self.errorrenderer = wrap_error_renderer
        self.errorrenderer_args = kw_args


class FormularpageController(MpulsFormularpageController):

    def _get_formpage_renderer(self):
        return CoachesPageRenderer().get_rendered_page
