# -*- coding: utf-8 -*-
#
# Copyright 2018 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls JMD has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.

from cgi import escape

from mpulsweb.lib.helpers import url, format_object
from mpulsweb.lib.helper.filters import NA

from mpulsweb.controllers.case_bundle import \
    CaseBundleController as MpulsCaseBundleController


def build_case_list(cases):
    """JMD specific version of mpulsweb.controllers.case_bundle.build_case_list.

    The difference is that we hard-code the fields we use here whereas
    the other build_case_list function re-uses the list of overview
    fields. That doesn't work well in JMD because we now have more
    fields than just the name. Some of those fields are not very useful
    without an indication of what they actually are (e.g. it would now
    attempt to show the age without telling you that it's the age) or
    fields that need special treatment, like fields read from
    repeatgroups.
    """
    list = ["<ul>"]
    fields = ("name", "vorname")
    for case in cases:
        list.append("<li>%s</li>"
                    % ", ".join([escape(format_object(NA(case.get_value(field))))
                                 for field in fields]))
    list.append("</ul>")
    return list


class CaseBundleController(MpulsCaseBundleController):

    def _buildCaseList(self, case_bundle):
        case_list = case_bundle.getCases()
        return build_case_list(case_list)
