"""Functions to import modules from a specialization or the base"""

import sys
import logging

from paste.util.import_string import try_import_module, import_module

from pylons import config as pylons_config


log = logging.getLogger(__name__)


def import_overridable_module(modulename, config=None):
    """Import a module that an MPuls specialization can override.

    The modulename should be a name relative to the mpulsweb package,
    e.g. 'controllers.foo' for the foo controller.  This function first
    tries to import the module from the specialization's main package,
    as configured with the mpuls.app.instance setting.  If that fails
    the module will be imported from the mpulsweb package.

    If the module can be imported successfully, it will be returned.  If
    it cannot be imported from either location, an exception will be
    raised (most likely an ImportError).
    """

    if config is None:
        config = pylons_config
    # Append root dir of the application configuration to the pythonpath
    app_root = config.get('mpuls.app.root')
    if app_root not in sys.path:
        sys.path.append(app_root)

    instance_module = config.get('mpuls.app.instance') + "." + modulename
    module = try_import_module(instance_module)
    if module is not None:
        return module

    base_module = "mpulsweb." + modulename
    log.debug("Module %r doesn't seem to exist, importing %r instead",
              instance_module, base_module)
    return import_module(base_module)

