# -*- coding: utf-8 -*-
import logging

from pylons import tmpl_context as c, app_globals as g

from mpulsweb.model.news import NewsList
from mpulsweb.model.percentage import JobPercentage
from mpulsweb.model.appointment import GlobalAppointmentOverview, \
     CaseAppointmentOverview, MaxSaveTimeReminderOverview, \
     CaseReminderOverview, TagsReminderOverview
from mpulsweb.lib.base import BaseController, render, session
import mpulsweb.lib.helpers as h


log = logging.getLogger(__name__)


class MpulsController(BaseController):

    def index(self):
        user = session['USER_AUTHORIZED']

        # LOAD NEWS
        if g.mpuls_config.is_enabled('module', 'news'):
            log.debug('Loading news list')
            c.news = self.get_newslist(user)

        # CALC JOB PERCENATAGE VALUE AND GENERATE WARING
        if g.mpuls_config.is_enabled('job_percentage', 'enabled'):
            c.threshold_higher = float(g.mpuls_config.get('job_percentage',
                                                          'threshold_higher'))
            c.threshold_lower = float(g.mpuls_config.get('job_percentage',
                                                         'threshold_lower'))
            c.percentage_value = self.get_jobpercentage_value()

        # LOAD APPOINTMENTS
        if g.mpuls_config.is_enabled('case-module', 'appointments'):
            # Case appointments
            log.debug('Loading case appointment list')
            c.appointmentlist = self.get_case_appointments()
        if g.mpuls_config.is_enabled('case-module', 'reminders'):
            # Case reminders
            log.debug('Loading reminders list')
            c.remindlist = self.get_reminders()
            c.show_set_done = True
        if g.mpuls_config.is_enabled('module', 'appointments'):
            # Global appointments
            log.debug('Loading global appointment list')
            c.globalappointmentlist = self.get_global_appointments()

        self.apply_hooks()

        return render('/home/home.mako')

    def apply_hooks(self):
        '''Perform actions on the cases the the blackboard is loaded. E.g
        create new RG instances etc. Should be overwritten in specific
        application.'''
        pass

    def get_newslist(self, user):
        """Return an agency wide newslist.
        The user parameter is ignored. It's only present for backwards
        compatibility. It effectively defaults to the currently logged in user.
        Only news not yet marked as read by the current user is
        contained in the returned newslist.
        """
        return NewsList()

    def get_global_appointments(self):
        """Return agency wide appointments"""
        return GlobalAppointmentOverview()

    def get_case_appointments(self):
        """Return the user's appointments and reminders"""
        appointment_list = CaseAppointmentOverview()
        return appointment_list

    def get_jobpercentage_value(self):
        percentage = JobPercentage()
        cases, percentage_value = percentage.calulate_percentage()
        return percentage_value

    def get_reminders(self):
        """Return an AppointmentOverview instance with reminders.
        The overview always contains the max save time reminders.
        """
        remindlist = MaxSaveTimeReminderOverview()
        if h.hasRole(['cm']):
            remindlist.join(CaseReminderOverview())

        if h.hasRole(['cm']):
            remindlist.join(TagsReminderOverview())

        remindlist.sort()

        return remindlist
