# -*- coding: utf-8 -*-
import logging
import formencode

from pylons.controllers.util import abort

from pylons import app_globals as g, tmpl_context as c, request
from mpulsweb.lib.base import BaseController, render, render_htmlfill, \
     restrict_to_POST, session
from mpulsweb.lib.reportquota import ReportQuotaData, SiteReportQuotaData
from mpulsweb.lib.security import checkRole
from mpulsweb.lib.validators import YearForm

log = logging.getLogger(__name__)

class ReportquotaController(BaseController):

    '''Display the reportquota of the agencies.'''

    @checkRole(('cm','admin'))
    def index(self):
        if g.mpuls_config.is_enabled('module', 'meta-show-reportquota'):
            c.form_errors= {}
            return render("/reportquota/selection.mako")
        else:
            abort(404)

    @checkRole(('cm', 'admin'))
    @restrict_to_POST
    def table(self):
        if g.mpuls_config.is_enabled('module', 'meta-show-reportquota'):
            validator = YearForm()
            c.form_errors = {}
            try:
                c.form_result = validator.to_python(request.params)
                if g.mpuls_config.is_enabled('module',
                                       'meta-show-reportquota-evaluation'):
                    rqd = ReportQuotaData(c.form_result.get('year', 2013))
                    c.site_rqd = SiteReportQuotaData(rqd,
                                      session.get('evaluation_ids', []))
                    return render("/reportquota/reportquota-evaluation.mako")
                else:
                    c.report_quota_data = ReportQuotaData(c.form_result.get('year',2013))
                    return render("/reportquota/reportquota.mako")
            except formencode.Invalid, error:
                c.form_result = error.value
                c.form_errors = error.error_dict or {}
                form = render('/reportquota/selection.mako')
                return render_htmlfill(form, c.form_result, c.form_errors,
                                       auto_insert_errors=False)
        else:
            abort(404)
